import time


class LocalCache:
    """
    本地缓存
    """

    def __init__(self):
        self._cache = {}

    def set(self, key, value, expire=0):
        """设置缓存
        expire： -1为不会过期，>=0是过期时间，单位秒
        """
        self._cache.update({
            key: {
                "data": value,
                "expire": self._get_expired_time(expire)
            }
        })

    def get(self, key):
        result = self._cache.get(key)
        if result:
            return result
        return None

    @staticmethod
    def is_expired(result: dict):
        """是否过期"""
        if result.get("expire", 0) > int(time.time()):
            return False
        return True

    @staticmethod
    def _get_expired_time(expire: int):
        """获取合法过期时间"""
        return int(time.time()) + expire if expire >= 0 else int(time.time())


local_cache = LocalCache()



import json
import redis

from pressure_test_lib.base.constants import PressureTestRedisConfig, PressureTestConfig
from pressure_test_lib.cache.cache import local_cache


def get_redis_connection():
    """获取一个redis连接"""
    if PressureTestRedisConfig.conn is None:
        connection_pool = redis.ConnectionPool(
            host=PressureTestRedisConfig.host,
            port=PressureTestRedisConfig.port,
            password=PressureTestRedisConfig.password,
            db=PressureTestRedisConfig.db,
            socket_timeout=PressureTestRedisConfig.timeout,
            max_connections=PressureTestRedisConfig.maxsize,
            decode_responses=True
        )
        PressureTestRedisConfig.conn = redis.Redis(connection_pool=connection_pool)

    return PressureTestRedisConfig.conn


def get_redis_mock_data_config():
    """获取redis里的mock数据"""
    conn = get_redis_connection()
    mock_data_config = conn.hgetall(PressureTestRedisConfig.mock_data_redis_key)
    return mock_data_config


def init_cache_mock_data():
    """初始化mock_data的数据进内存"""
    if not PressureTestConfig.switch:
        # 如果不是压测命名空间，就不初始化mock数据
        return None
    redis_mock_data_config = get_redis_mock_data_config()
    local_cache.set(PressureTestConfig.mock_cache_key, redis_mock_data_config, PressureTestConfig.mock_cache_ex)
    return local_cache.get(PressureTestConfig.mock_cache_key)



import json

from pressure_test_lib.base.constants import PressureTestRedisConfig, PressureTestConfig
from pressure_test_lib.cache.cache import local_cache

try:
    from aioredis import create_redis_pool
except:
    raise Exception('未安装 aioredis 库，支持 1.X.X 版本')


async def get_redis_connection():
    """获取一个redis连接"""
    if PressureTestRedisConfig.conn:
        return PressureTestRedisConfig.conn

    PressureTestRedisConfig.conn = await create_redis_pool(
        address=[PressureTestRedisConfig.host, PressureTestRedisConfig.port],
        db=PressureTestRedisConfig.db,
        password=PressureTestRedisConfig.password,
        minsize=PressureTestRedisConfig.minsize,
        maxsize=PressureTestRedisConfig.maxsize,
        timeout=PressureTestRedisConfig.timeout,
        encoding='utf-8',
        **(PressureTestRedisConfig.kwargs or {})
    )

    return PressureTestRedisConfig.conn


async def get_redis_mock_data_config():
    """获取redis里的mock数据"""
    conn = await get_redis_connection()
    mock_data_config = await conn.hgetall(PressureTestRedisConfig.mock_data_redis_key)
    return mock_data_config


async def init_cache_mock_data():
    """初始化mock_data的数据进内存"""
    if not PressureTestConfig.switch:
        # 如果不是压测命名空间，就不初始化mock数据
        return None
    redis_mock_data_config = await get_redis_mock_data_config()
    await local_cache.set(PressureTestConfig.mock_cache_key, redis_mock_data_config, PressureTestConfig.mock_cache_ex)
    return await local_cache.get(PressureTestConfig.mock_cache_key)



from pressure_test_lib.base.constants import PressureTestConfig
from pressure_test_lib.cache.cache import local_cache
from pressure_test_lib.request_g.mock_data import init_cache_mock_data
from pressure_test_lib.base.tools import get_mock_data

try:
    import requests.adapters
except ImportError:
    pass
else:
    _HTTPAdapter_send = requests.adapters.HTTPAdapter.send
    from requests import Response


def mock_response(mock_data):
    """mock一个response对象"""
    response = Response()
    response.status_code = 200
    response._content = bytes(mock_data, 'utf-8')
    response.headers = 'application/json'
    return response


def requests_send_wrapper(http_adapter, request, **kwargs):
    """Wraps HTTPAdapter.send"""
    if PressureTestConfig.switch:
        # 压测配置是否开启，用来判断是否为压测命名空间
        mock_data_cache = local_cache.get(PressureTestConfig.mock_cache_key)
        if local_cache.is_expired(mock_data_cache):
            # 如果内存数据过期，则再从redis取数据存进内存
            mock_data_cache = init_cache_mock_data()
        method, url = request.method, request.url
        mock_data = get_mock_data(mock_data_cache, url)
        if mock_data:
            return mock_response(mock_data)

    response = _HTTPAdapter_send(http_adapter, request, **kwargs)
    return response


def patch_requests():
    if '_HTTPAdapter_send' not in globals():
        raise Exception("requests not installed.")
    requests.adapters.HTTPAdapter.send = requests_send_wrapper



from urllib.parse import urlparse

from pressure_test_lib.base.constants import PressureTestConfig


def url_to_domain_path(url):
    """从url获取域名和uri"""
    url = urlparse(url)
    domain = url.hostname
    path = url.path
    return domain, path


def get_mock_data(mock_data_cache, url):
    """获取mock数据"""
    data = mock_data_cache.get('data', {})
    domain_name, uri = url_to_domain_path(url)
    mock_data = data.get('%s%s' % (domain_name, uri))
    return mock_data



"""常量"""


class PressureTestConfig:
    """压测配置"""

    # 开关
    switch = 0

    # 压测mock数据内存key
    mock_cache_key = 'mock_cache_key'

    # 内存过期时间
    mock_cache_ex = 60

    # 服务名称
    service_name = ''


class PressureTestRedisConfig:
    """压测redis配置"""
    # 压测所属服务
    service = 'default'

    # 地址
    host = '127.0.0.1'

    # 端口
    port = 6379

    # 数据库
    db = 0

    # 密码
    password = None

    # 最小连接数
    minsize = 1

    # 最大连接数
    maxsize = 10

    # 超时时间【毫秒】
    timeout = 5000

    # 额外参数
    kwargs = None

    # 初始化连接池
    conn = None

    # mock数据的redis_key
    mock_data_redis_key = 'mock_data'



import setuptools
from os import path as os_path

this_directory = os_path.abspath(os_path.dirname(__file__))


def read_file(filename):
    with open(os_path.join(this_directory, filename), encoding='utf-8') as f:
        long_description = f.read()
    return long_description


with open("README.md", "r") as fh:
    long_description = fh.read()


def read_requirements(filename):
    return [
        line.strip() 
        for line in read_file(filename).splitlines() 
        if not line.startswith('#')
    ]


setuptools.setup(
    name="pressure_test_lib",
    version="0.0.1",
    author="guanmengshi",
    description="压测包",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/guanmengshi/test",
    keywords=['test', 'pressure-test'],
    packages=setuptools.find_packages(),
    install_requires=read_requirements('requirements.txt'),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
)



import json
import pytest
from pressure_test_lib.base.constants import PressureTestConfig
from pressure_test_lib.base.tools import get_mock_data
from pressure_test_lib.cache.cache import local_cache
from pressure_test_lib.fastapi_test.mock_data import init_cache_mock_data

try:
    import aiohttp
except ImportError:
    pass
else:
    _client_session_request = aiohttp.ClientSession._request
    from aiohttp.client_reqrep import ClientResponse
    from aiohttp import http
    from multidict import CIMultiDict, CIMultiDictProxy
    from aiohttp import RequestInfo
    from unittest import mock
    from aiohttp.helpers import TimerNoop
    from yarl import URL
    import asyncio


@pytest.fixture
def session():
    return mock.Mock()


def mock_response(method, mock_data):
    """mock一个response对象"""
    headers = CIMultiDict()
    headers['Content-Type'] = 'application/json'
    headers = CIMultiDictProxy(headers)
    request_info = RequestInfo('http://pressure_test', "get", headers, 'http://pressure_test')
    loop = asyncio.get_event_loop()
    response = ClientResponse(
        method,
        URL("http://pressure_test"),
        request_info=request_info,
        writer=mock.Mock(),
        continue100=None,
        timer=TimerNoop(),
        traces=[],
        loop=loop,
        session=session,
    )

    def side_effect():
        fut = loop.create_future()
        fut.set_result(mock_data.encode("utf-8"))
        return fut

    response._headers = {"Content-Type": "application/json"}
    content = response.content = mock.Mock()
    content.read.side_effect = side_effect
    response.status = 200
    return response


async def client_session_request_wrapper(*args, **kwargs):
    """Wraps aiohttp.ClientSession._request"""
    if PressureTestConfig.switch:
        # 压测配置是否开启，用来判断是否为压测命名空间
        mock_data_cache = local_cache.get(PressureTestConfig.mock_cache_key)
        if mock_data_cache:
            # 从内存获取mock数据，内存中没有直接发起真实请求，内存里存的mock数据结构 {'data':{'domain_name_uri':mock_data}}
            if local_cache.is_expired(mock_data_cache):
                # 如果内存数据过期，则再从redis取数据存进内存
                mock_data_cache = await init_cache_mock_data()
            method, url = args[1], args[2]
            mock_data = get_mock_data(mock_data_cache, url)
            if mock_data:
                # 如果内存中没数据就直接发起请求
                return mock_response(method, mock_data)

    response = await _client_session_request(*args, **kwargs)
    return response


def patch_requests():
    if '_client_session_request' not in globals():
        raise Exception("aiohttp not installed.")
    aiohttp.ClientSession._request = client_session_request_wrapper
